@@ -1,8 +1,8 @@
 ############## last change 2012/03/16 #####################
 
 .Contlcmm <-
-function(fixed,mixture,random,subject,classmb,ng,idiag,nwg,data,B,convB,convL,convG,prior,maxiter,epsY,idlink0,ntrtot0,nbzitr0,zitr,nsim,call,Ydiscrete,na.action){
-
+function(fixed,mixture,random,subject,classmb,ng,idiag,nwg,cor,data,B,convB,convL,convG,prior,maxiter,epsY,idlink0,ntrtot0,nbzitr0,zitr,nsim,call,Ydiscrete,subset,na.action){
+  
 cl <- match.call()
 
 args <- as.list(match.call(.Contlcmm))[-1]
@@ -31,6 +31,25 @@ if(class(classmb)!="formula") stop("The argument classmb must be a formula")
 if(missing(data)){ stop("The argument data should be specified and defined as a data.frame")} 
 if(missing(subject)){ stop("The argument subject must be specified in any model even without random-effects")} 
 
+### test de l'argument cor
+ncor0 <- 0    
+if (!is.null(cor))
+{
+ #if (typeof(cor)!= "character") stop("The argument cor must be a character")
+
+ if (substr(cor,1,2)=="AR") { ncor0 <- 2 }
+ else if (substr(cor,1,2)=="BM") { ncor0 <- 1  }
+ else { stop("The argument cor must be of type AR or BM") }
+ 
+ #if (substr(cor,3,3)!="-") stop("Invalid argument cor")
+ #if (grep("-",cor)>1) stop("Invalid argument cor")
+ 
+ if(!(strsplit(cor,"-")[[1]][2] %in% colnames(data))) stop("Unaible to find time variable from argument cor in data")
+ else { cor.var.time <- strsplit(cor,"-")[[1]][2] }
+}  
+### fin test argument cor 
+
+
 ### ad 2/04/2012
 X0.names2 <- c("intercept")
 ### ad
@@ -39,18 +58,25 @@ int.mixture <- 0
 int.random <- 0
 int.classmb <- 0
 
-### Traitement des donnees manquantes
-if(!is.null(na.action)){
-	newdata <- data[-na.action,]
-}else{
-	newdata <- data
-}
+           
+## Table sans donnees manquante: newdata
+
+newdata <- data  
+#prendre le subset :
+if(!(is.null(subset))) 
+newdata <- data[subset,]
+
+#enlever les NA
+	if(!is.null(na.action)){
+		newdata <- newdata[-na.action,]
+	}
 
 ### names of covariate in intial fit
 X0.names2 <- unique(c(X0.names2,colnames(get_all_vars(formula(terms(fixed)),data=newdata))[-1]))
 if(mixture[[2]] != "-1")X0.names2 <- unique(c(X0.names2,colnames(get_all_vars(formula(terms(mixture)),data=newdata))))
 if(random[[2]] != "-1")X0.names2 <- unique(c(X0.names2,colnames(get_all_vars(formula(terms(random)),data=newdata))))
 if(classmb[[2]] != "-1")X0.names2 <- unique(c(X0.names2,colnames(get_all_vars(formula(terms(classmb)),data=newdata))))
+if(ncor0>0) X0.names2 <- unique(c(X0.names2,cor.var.time))
 
 ## Construction de nouvelles var eplicatives sur la nouvelle table
 ## fixed
@@ -114,6 +140,11 @@ if(id.X_mixture == 1) var.exp <- c(var.exp,colnames(X_mixture))
 if(id.X_random == 1)var.exp <- c(var.exp,colnames(X_random))
 if(id.X_classmb == 1)var.exp <- c(var.exp,colnames(X_classmb))
 var.exp <- unique(var.exp)
+if(ncor0>0) 
+{ if(!(cor.var.time %in% var.exp)) 
+  {var.exp <- c(var.exp, cor.var.time)} #si la varaible de temps dans cor n'est dan sles variables expl, on l'ajoute
+}
+
 ## ad
 
 
@@ -131,7 +162,7 @@ if(id.X_mixture == 1){
 		if((colnames(X_mixture)[i] %in% colnames(X0))==F){
 			X0 <- cbind(X0,X_mixture[,i])
 			
-		}
+		}                                                                
 	}
 }
 if(id.X_random == 1){
@@ -148,6 +179,14 @@ if(id.X_classmb == 1){
 		}	
 	}
 }
+
+if(ncor0>0) 
+{ if(!(cor.var.time %in% colnames(X0))) 
+  {
+   X0 <- cbind(X0, newdata[,cor.var.time])
+  }
+}  
+
 colnames(X0) <- var.exp
 
 if((any(is.na(X0))==TRUE)|(any(is.na(Y0))==TRUE))stop("The data should not contain any missing value")
@@ -160,8 +199,8 @@ if (!((int.fixed+int.random)>0)) X0 <- as.data.frame(X0[,-which(colnames(X0)=="i
 
 nom.X0 <- colnames(X0)
 nvar.exp <- length(nom.X0)
-
-IND <- newdata[,nom.subject]
+      
+IND <- newdata[,nom.subject] 
 IDnum <- as.numeric(IND)
 
 
@@ -180,7 +219,7 @@ nwg0 <- as.integer(nwg)
 idea0 <- rep(0,nvar.exp)
 idprob0 <- rep(0,nvar.exp)
 idg0 <- rep(0,nvar.exp)
-
+idcor0 <- rep(0,nvar.exp)
 
 for (i in 1:nvar.exp)    {
  idea0[i] <- nom.X0[i]%in%inddepvar.random.nom
@@ -188,6 +227,9 @@ for (i in 1:nvar.exp)    {
  if(nom.X0[i]%in%nom.fixed & !(nom.X0[i]%in%nom.mixture)) idg0[i] <- 1 
  if(nom.X0[i]%in%nom.fixed & nom.X0[i]%in%nom.mixture) idg0[i] <- 2  
  }
+ 
+if (ncor0!=0) idcor0 <- as.numeric(nom.X0 %in% cor.var.time)
+ 
 if((int.fixed+int.random)>0) idprob0[1] <- 0
 
 # on ordonne les donn es suivants la variable IND
@@ -267,9 +309,10 @@ b1[(NEF+1):(NEF+NVC)]<-bidiag
 }
 
 # valeurs initiales pour les transfos #
-if (idlink0==0) 
+if (idlink0==0){ 
 b1[NEF+NVC+1]<-mean(Y0)
 b1[NEF+NVC+2]<-1
+}
 if (idlink0==1){
 b1[(NEF+NVC+1)]<-0
 b1[(NEF+NVC+2)]<- -log(2)
@@ -281,6 +324,11 @@ b1[(NEF+NVC+1)]<- -2
 b1[(NEF+NVC+2):(NEF+NVC+ntrtot0)]<- 0.1
 }
 
+if(ncor0==1)
+{b1[NEF+NVC+ntrtot0+1] <- 1 }
+if(ncor0==2)
+{b1[(NEF+NVC+ntrtot0+1):(NEF+NVC+ntrtot0+ncor0)] <- c(0,1) }
+
 
 NPM<-length(b1)
 NW<-0
@@ -298,8 +346,12 @@ kk<-sum(idea0==1)
 NVC<-(kk*(kk+1))/2}
 NW<-nwg0*(ng0-1)
 if(NW>0) b[(NPROB+NEF+NVC+1):(NPROB+NEF+NVC+NW)]<-1
-NPM<-NPROB+NEF+NVC+NW+ntrtot0
-V <- rep(0,NPM*(NPM+1)/2)
+if(ncor0==1)
+{b[NEF+NVC+NW+ntrtot0+1] <- 1 }
+if(ncor0==2)
+{b[(NEF+NVC+NW+ntrtot0+1):(NEF+NVC+NW+ntrtot0+ncor0)] <- c(0,1) }
+NPM<-NPROB+NEF+NVC+NW+ntrtot0+ncor0
+V <- rep(0,NPM*(NPM+1)/2) 
 } 
 
 
@@ -311,21 +363,45 @@ idea2 <- idea0
 idprob2 <- rep(0,nv0)  
 idg2 <- rep(0,nv0) 
 idg2[idg0!=0] <- 1
-NEF2<-sum(idg2==1)-1
-NPM2<-NEF2+NVC+ntrtot0
-nwg2<-0
-ng2<-1
-ppi2<- rep(0,ns0)
+NEF2 <- sum(idg2==1)-1
+NPM2 <- NEF2+NVC+ntrtot0+ncor0
+nwg2 <- 0
+ng2 <- 1
+ppi2 <- rep(0,ns0)
 pred_m_g2 <- rep(0,nobs0)
 pred_ss_g2 <- rep(0,nobs0)
+maxiter2 <- min(75,maxiter)
+convB2 <- max(0.01,convB)
+convL2 <- max(0.01,convL)
+convG2 <- max(0.01,convG)
 
 V2 <- rep(0,NPM2*(NPM2+1)/2)
 
 marker <- rep(0,nsim)
 transfY <- rep(0,nsim)
-init <- .Fortran("hetmixCont",as.double(Y0),as.double(X0),as.integer(prior2),as.integer(idprob2),as.integer(idea2),as.integer(idg2),as.integer(ns0),as.integer(ng2),as.integer(nv0),as.integer(nobs0),as.integer(nea0),as.integer(nmes0),as.integer(idiag0),as.integer(nwg2),as.integer(NPM2),best=as.double(b1),V=as.double(V2),as.double(loglik),niter=as.integer(ni),as.integer(istop),as.double(gconv),as.double(ppi2),as.double(resid_m),as.double(resid_ss),as.double(pred_m_g2),as.double(pred_ss_g2),predRE=as.double(predRE),as.double(convB),as.double(convL),as.double(convG),as.integer(maxiter),as.double(epsY),as.integer(idlink0),as.integer(nbzitr0),as.double(zitr),as.double(marker),as.double(transfY),as.integer(nsim),as.double(Yobs),as.integer(Ydiscrete),as.double(vraisdiscret),as.double(UACV),as.double(rlindiv),PACKAGE="lcmm")
+
+#cat("parametres pour init : \n")
+#cat("nv=",nv0,"\n")
+#cat("idprob, idea, idg,idcor",idprob2,idea2,idg2,idcor0,"\n")
+#cat("b1",b1,"\n")
+#cat("npm",NPM2,"\n")
+#cat("NEF2+NVC+ntrtot0+ncor0=",NEF2,"+",NVC,"+",ntrtot0,"+",ncor0,"\n")
+
+init <- .Fortran("hetmixCont",as.double(Y0),as.double(X0),as.integer(prior2),
+as.integer(idprob2),as.integer(idea2),as.integer(idg2),as.integer(idcor0),as.integer(ns0),
+as.integer(ng2),as.integer(nv0),as.integer(nobs0),as.integer(nea0),as.integer(nmes0),
+as.integer(idiag0),as.integer(nwg2),as.integer(ncor0),as.integer(NPM2),best=as.double(b1),
+V=as.double(V2),as.double(loglik),niter=as.integer(ni),conv=as.integer(istop),as.double(gconv),
+as.double(ppi2),as.double(resid_m),as.double(resid_ss),as.double(pred_m_g2),as.double(pred_ss_g2),
+predRE=as.double(predRE),as.double(convB2),as.double(convL2),as.double(convG2),as.integer(maxiter2),
+as.double(epsY),as.integer(idlink0),as.integer(nbzitr0),as.double(zitr),as.double(marker),
+as.double(transfY),as.integer(nsim),as.double(Yobs),as.integer(Ydiscrete),as.double(vraisdiscret),
+as.double(UACV),as.double(rlindiv),PACKAGE="lcmm")
 
 
+#cat("\n")
+#cat("cv de init :",init$conv,"\n")
+#cat("estimation de init",init$best,"\n")
 
 k <- NPROB
 l <- 0
@@ -348,14 +424,16 @@ if (i>1){
 	l <- l+1
 	for (g in 1:ng){
 	t <- t+1
-	b[k+t] <- init$best[l]+(g-(ng+1)/2)*sqrt(init$V[l*(l+1)/2])
+	if(init$conv==1) b[k+t] <- init$best[l]+(g-(ng+1)/2)*sqrt(init$V[l*(l+1)/2])
+	else b[k+t] <- init$best[l]+(g-(ng+1)/2)*init$best[l]
 	}
 }
 }
 }
 
 b[(NPROB+NEF+1):(NPROB+NEF+NVC)] <-init$best[(NEF2+1):(NEF2+NVC)]
-b[(NPM-ntrtot0+1):NPM] <-init$best[(NPM2-ntrtot0+1):NPM2]
+b[(NPM-ntrtot0+1):(NPM-ncor0)] <-init$best[(NPM2-ntrtot0+1):(NPM2-ncor0)]
+if (ncor0>0) {b[(NPROB+NEF+NVC+NW+ntrtot0+1):(NPROB+NEF+NVC+NW+ntrtot0+ncor0)] <- init$best[(NPM2-ncor0+1):NPM2]}
 } 
 if(ng0==1 ){
 b <- b1
@@ -365,6 +443,8 @@ else {if(length(B)!=NPM)stop("The length of the vector B is not correct")
  else {b <-B}
 }
 
+
+
 se <- rep(0,length(b))
 #------------------------------------------
 #------nom au vecteur best
@@ -397,26 +477,29 @@ if(ng0>1){
 names(b)[(NPROB+1):(NPROB+NEF)]<- nom1
 }
 
-if(idlink0==0) names(b)[(NPM-ntrtot0+1):NPM]<- c("Linear 1 (intercept)","Linear 2 (std err)")
-if(idlink0==1) names(b)[(NPM-ntrtot0+1):NPM]<- paste("Beta",c(1:ntrtot0),sep="")
-if(idlink0==2) names(b)[(NPM-ntrtot0+1):NPM]<- paste("I-splines",c(1:ntrtot0),sep="")
+if(idlink0==0) names(b)[(NPM-ntrtot0+1-ncor0):(NPM-ncor0)]<- c("Linear 1 (intercept)","Linear 2 (std err)")
+if(idlink0==1) names(b)[(NPM-ntrtot0+1-ncor0):(NPM-ncor0)]<- paste("Beta",c(1:ntrtot0),sep="")
+if(idlink0==2) names(b)[(NPM-ntrtot0+1-ncor0):(NPM-ncor0)]<- paste("I-splines",c(1:ntrtot0),sep="")
 
 
 if(NVC!=0)names(b)[(NPROB+NEF+1):(NPROB+NEF+NVC)] <- paste("varcov",c(1:(NVC)))
 if(NW!=0)names(b)[(NPROB+NEF+NVC+1):(NPROB+NEF+NVC+NW)] <- paste("varprop class",c(1:(ng0-1)))
 
+if (ncor0>0) {names(b)[(NPROB+NEF+NVC+NW+ntrtot0+1):(NPROB+NEF+NVC+NW+ntrtot0+ncor0)] <- paste("cor",1:ncor0,sep="")}
+
 N <- NULL
 N[1] <- NPROB
 N[2] <- NEF
 N[3] <- NVC
 N[4] <- NW
 N[5] <- nobs0
+N[6] <- ncor0
 
 idiag <- as.integer(idiag0)
 idea <- as.integer(idea0)
 nv <- as.integer(nv0)
 
-
+#cat("valeurs initiales : ",b,"\n")
 ################ Sortie ###########################
 ptm<-proc.time()
 cat("Be patient, lcmm is running ... \n")
@@ -433,9 +516,20 @@ transfY <- rep(0,nsim)
 #cat(paste("B:"),"\n")
 #print(b)
 
-out <- .Fortran("hetmixCont",as.double(Y0),as.double(X0),as.integer(prior0),as.integer(idprob0),as.integer(idea0),as.integer(idg0),as.integer(ns0),as.integer(ng0),as.integer(nv0),as.integer(nobs0),as.integer(nea0),as.integer(nmes0),as.integer(idiag0),as.integer(nwg0),as.integer(NPM),best=as.double(b),V=as.double(V),loglik=as.double(loglik),niter=as.integer(ni),conv=as.integer(istop),gconv=as.double(gconv),ppi2=as.double(ppi0),resid_m=as.double(resid_m),resid_ss=as.double(resid_ss),pred_m_g=as.double(pred_m_g),pred_ss_g=as.double(pred_ss_g),predRE=as.double(predRE),as.double(convB),as.double(convL),as.double(convG),as.integer(maxiter),as.double(epsY),as.integer(idlink0),as.integer(nbzitr0),as.double(zitr),marker=as.double(marker),transfY=as.double(transfY),as.integer(nsim),Yobs=as.double(Yobs),as.integer(Ydiscrete),vraisdiscret=as.double(vraisdiscret),UACV=as.double(UACV),rlindiv=as.double(rlindiv),PACKAGE="lcmm")
 
+out <- .Fortran("hetmixCont",as.double(Y0),as.double(X0),as.integer(prior0),as.integer(idprob0),
+as.integer(idea0),as.integer(idg0),as.integer(idcor0),as.integer(ns0),as.integer(ng0),
+as.integer(nv0),as.integer(nobs0),as.integer(nea0),as.integer(nmes0),as.integer(idiag0),
+as.integer(nwg0),as.integer(ncor0),as.integer(NPM),best=as.double(b),V=as.double(V),
+loglik=as.double(loglik),niter=as.integer(ni),conv=as.integer(istop),gconv=as.double(gconv),
+ppi2=as.double(ppi0),resid_m=as.double(resid_m),resid_ss=as.double(resid_ss),
+pred_m_g=as.double(pred_m_g),pred_ss_g=as.double(pred_ss_g),predRE=as.double(predRE),
+as.double(convB),as.double(convL),as.double(convG),as.integer(maxiter),as.double(epsY),
+as.integer(idlink0),as.integer(nbzitr0),as.double(zitr),marker=as.double(marker),
+transfY=as.double(transfY),as.integer(nsim),Yobs=as.double(Yobs),as.integer(Ydiscrete),
+vraisdiscret=as.double(vraisdiscret),UACV=as.double(UACV),rlindiv=as.double(rlindiv),PACKAGE="lcmm")
 
+#cat("fin de fortran \n")
 
 ### Creation du vecteur cholesky
 Cholesky <- rep(0,(nea0*(nea0+1)/2))
@@ -496,7 +590,7 @@ colnames(estimlink) <- c("Y","transfY")
 ### ad 2/04/2012
 if (!("intercept" %in% nom.X0)) X0.names2 <- X0.names2[-1]
 ### ad
-res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,loglik=out$loglik,best=out$best,V=out$V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action)
+res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=out$V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action)
 class(res) <-c("lcmm") 
 
 cost<-proc.time()-ptm
