@@ -1,5 +1,9 @@
-plot.Diffepoce <- function(x,...){
+plot.Diffepoce <- function(x,ylim=NULL,xlim=NULL,...){
 	if (!inherits(x, "Diffepoce")) stop("use only with \"Diffepoce\" objects")
+
+	if (is.null(ylim)&all(is.na(x$DiffEPOCE[,3]))|all(is.na(x$DiffEPOCE[,4]))) stop("can't produce the plot with missing differences in EPOCE")
+
+
       dev.new()
 
 	if(x$new.data==FALSE) {
@@ -8,10 +12,14 @@ plot.Diffepoce <- function(x,...){
 		title <- expression(Delta(MPOL))	
 	}
 
-
+ylim1 <- ylim
+if (is.null(ylim)){
 ylim1 <- c(min(x$DiffEPOCE[!(is.na(x$DiffEPOCE[,2])),2],x$DiffEPOCE[!(is.na(x$DiffEPOCE[,3])),3]),max(x$DiffEPOCE[!(is.na(x$DiffEPOCE[,2])),2],x$DiffEPOCE[!(is.na(x$DiffEPOCE[,4])),4]))	
-
+}
+xlim1 <- xlim
+if (is.null(xlim)){
 xlim1 <- c(min(x$DiffEPOCE[,1]),max(x$DiffEPOCE[,1]))
+}
 
 plot(x$DiffEPOCE[,1],x$DiffEPOCE[,2],pch=18,lty=1,type="o",xlab="prediction time",ylab=title,main="Difference in EPOCE estimates",bty="l",ylim=ylim1,xlim=xlim1,...)
 par(new=T)
